/*****************************************************************************
 * file    lib/process/ca_main.c  
 * author  YuLiang
 * version 1.0.0
 * date    27-Nov-2023
 * brief   This file provides all the cable related operation functions.
 ******************************************************************************
 * Attention
 *
 * <h2><center>&copy; COPYRIGHT(c) 2023 LandPower</center></h2>
 *
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 *   1. Redistributions of source code must retain the above copyright notice,
 *      this list of conditions and the following disclaimer.
 *   2. Redistributions in binary form must reproduce the above copyright notice,
 *      this list of conditions and the following disclaimer in the documentation
 *      and/or other materials provided with the distribution.
 *   3. Neither the name of LandPower nor the names of its contributors may be used to 
 *      endorse or promote products derived from this software without specific
 *      prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 ******************************************************************************/

/* Includes ------------------------------------------------------------------*/
#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#ifdef CFG_DEV_TYPE_LAND_CA
/* 标准C库头文件. */

/* 私有头文件 */
#include "ca_db.h"
#include "ca_collect.h"
#include "ca_dbg.h"
#include "ca_mqtt.h"
#include "ca_land.h"

/* Private typedef -----------------------------------------------------------*/

/* Private define ------------------------------------------------------------*/

/* Private macro -------------------------------------------------------------*/

/* Private variables ---------------------------------------------------------*/


/* Private function prototypes -----------------------------------------------*/

/* Internal functions --------------------------------------------------------*/

/* 将端口字符串, 转换成 unit port 格式. */

/* Interface functions -------------------------------------------------------*/
/* description: 环流预初始化.
   param:
   return: E_XXX */
int32_t ca_main(void)
{
    int32_t rv = E_NONE;

    rv |= ca_db_init();
    rv |= ca_coll_init();
    rv |= ca_mqtt_init();
    rv |= ca_land_init();
    
    return rv;
}

/* description: 环流初始化
   param:
   return: E_XXX */
int32_t ca_main_after(void)
{
    int32_t rv = E_NONE;

    rv |= ca_db_init_after();
    rv |= ca_coll_init_after();
    rv |= ca_dbg_handle_init_after();
    rv |= ca_mqtt_init_after();
    rv |= ca_land_init_after();
    
    return rv;
}

#else
int32_t ca_main(void)
{
    return 0;
}

int32_t ca_main_after(void)
{
    return 0;
}
#endif
/************************ (C) COPYRIGHT LandPower ***** END OF FILE ****/

